/***************************************************************************
 * eqOSG - An example application which uses OpenSceneGraph with Equalizer *
 *                                                                         *
 * Copyright (C) 2008 by Thomas McGuire                                    *
 * thomas.mcguire@student.uni-siegen.de                                    *
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU Lesser General Public              *
 * License as published by the Free Software Foundation; either            *
 * version 2.1 of the License, or (at your option) any later version.      *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * Lesser General Public License for more details.                         *
 *                                                                         *
 * You should have received a copy of the GNU Lesser General Public        *
 * License along with this library; if not, write to the Free Software     *
 * Foundation, Inc., 51 Franklin Street,                                   *
 * Fifth Floor, Boston, MA  02110-1301  USA                                *
 ***************************************************************************/
#include "Application.h"

#include "Config.h"

#include <eq/client/server.h>
#include <eq/client/configParams.h>

using namespace std;

Application::Application( const InitData& initData )
    : mInitData( initData )
{
}

int Application::run()
{
    // 1. connect to server
    eq::ServerPtr server = new eq::Server;
    if( !connectServer( server ) )
    {
        std::cout << "Can't open server" << endl;
        return EXIT_FAILURE;
    }

    // 2. choose config
    eq::ConfigParams configParams;
    Config* config = static_cast<Config*>( server->chooseConfig( configParams ) );

    if( !config )
    {
        std::cout << "No matching config on server" << endl;
        disconnectServer( server );
        return EXIT_FAILURE;
    }

    config->setInitData( mInitData );
    if( !config->init() )
    {
        std::cout << "Config initialization failed: "
                  << config->getErrorMessage() << endl;
        server->releaseConfig( config );
        disconnectServer( server );
        return EXIT_FAILURE;
    }

    // 4. run main loop
    while( config->isRunning() )
    {
        config->startFrame();
        config->finishFrame();
    }
    config->finishAllFrames();

    // 5. exit config
    config->exit();

    // 6. cleanup and exit
    server->releaseConfig( config );
    if( !disconnectServer( server )) {
        std::cout << "Client::disconnectServer failed" << endl;
    }

    server = 0;

    return EXIT_SUCCESS;
}

bool Application::clientLoop()
{
    while( true )
    {
        if( !eq::Client::clientLoop() )
            return false;
    }
    return true;
}
