
/* Copyright (c) 2008-2009, Stefan Eilemann <eile@equalizergraphics.com> 
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 2.1 as published
 * by the Free Software Foundation.
 *  
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef EQSERVER_CONFIGVISITOR_H
#define EQSERVER_CONFIGVISITOR_H

#include "canvasVisitor.h"        // base class
#include "constCompoundVisitor.h" // base class
#include "compoundVisitor.h"      // base class
#include "layoutVisitor.h"        // base class
#include "nodeVisitor.h"          // base class
#include "observerVisitor.h"      // base class

namespace eq
{
namespace server
{
    class Config;

    /**
     * A visitor to traverse non-const configs and children.
     */
    class ConfigVisitor : public NodeVisitor, 
                          public CompoundVisitor,
                          public ObserverVisitor,
                          public LayoutVisitor,
                          public CanvasVisitor
    {
    public:
        /** Constructs a new ConfigVisitor. */
        ConfigVisitor(){}
        
        /** Destruct the ConfigVisitor */
        virtual ~ConfigVisitor(){}

        /** Visit a config on the down traversal. */
        virtual VisitorResult visitPre( Config* config )
            { return TRAVERSE_CONTINUE; }

        /** Visit a config on the up traversal. */
        virtual VisitorResult visitPost( Config* config )
            { return TRAVERSE_CONTINUE; }
    };

    /**
     * A visitor to traverse const configs and children.
     */
    class ConstConfigVisitor : public ConstNodeVisitor, 
                               public ConstCompoundVisitor,
                               public ConstObserverVisitor,
                               public ConstLayoutVisitor,
                               public ConstCanvasVisitor
    {
    public:
        /** Constructs a new ConfigVisitor. */
        ConstConfigVisitor(){}
        
        /** Destruct the ConfigVisitor */
        virtual ~ConstConfigVisitor(){}

        /** Visit a config on the down traversal. */
        virtual VisitorResult visitPre( const Config* config )
            { return TRAVERSE_CONTINUE; }

        /** Visit a config on the up traversal. */
        virtual VisitorResult visitPost( const Config* config )
            { return TRAVERSE_CONTINUE; }
    };
}
}
#endif // EQSERVER_CONFIGVISITOR_H
